local cooldowns = { }

local charging = { }

local EMPTY_TABLE = { }

local NEXT_PRUNE = 0

local HOLD_TIMER = 1.5

local id = 0

-- PABR_CD = cooldowns -- FOR DEBUG ONLY

local function newId()
	id = id + 1
	return id
end

local ctx = {
	guid = "",

	-- The following methods use variable length arguments joined by a 'OR' logical operation

	HasTalent = function(self, ...)
		for i=1, select('#', ...) do
			if PABR_HasTalent(self.guid, select(i, ...)) then return true end
		end
		return false
	end,

	HasGlyph = function(self, ...)
		for i=1, select('#', ...) do
			if PABR_HasGlyph(self.guid, select(i, ...)) then return true end
		end
		return false
	end,

	HasItemSet = function(self, threshold, ...)
		local count = 0
		for i=1, select('#', ...) do
			if PABR_HasItem(self.guid, select(i, ...)) then count = count + 1 end
		end
		return count >= threshold
	end,

	HasSpec = function(self, ...)
		local spec = PABR_GetSpecialization(self.guid)
		for i=1, select('#', ...) do
			if spec == select(i, ...) then return true end
		end
		return false
	end,
}

local function eval(f)
	if type(f) == "function" then
		return f(ctx)
	else
		return f or 0
	end
	assert(type(f) == "number")
end

local function detectMatch(firedMethod, method)
	return firedMethod == method or (firedMethod == "miss" and (method == "hit" or method == "effectup"))
end

local function findCooldown(guid, spellId)
	local list = cooldowns[guid] or EMPTY_TABLE
	for i=1, #list do
		local data = list[i]
		if data[1] == spellId then
			return i, data
		end
	end
	return 0, nil
end

local function insertCooldown(guid, spellId, timestamp, duration, maxCharge)
	local id = newId()
	
	local data = PABR_CreateTable()
	data[1] = spellId
	data[2] = PABR_GetTime(timestamp)
	data[3] = duration
	if maxCharge > 1 then
		charging[id] = data
		data[4] = maxCharge - 1
		data[5] = maxCharge
	else
		data[4] = 0
		data[5] = 0
	end
	data[6] = id

	local list = cooldowns[guid]
	if not list then
		list = PABR_CreateTable()
		cooldowns[guid] = list
	end

	tinsert(list, data)
end

local function updateCooldown(data, timestamp, duration)
	data[3] = duration
	if data[4] == data[5] then
		data[2] = PABR_GetTime(timestamp)
	end
	if data[5] > 0 then
		data[4] = data[4] - 1
	end
end

local function deleteCooldownSafe(list, index)
	local data = list[index]
	charging[data[6]] = nil
	PABR_DeleteTable(data)
	tremove(list, index)
end

local function deleteCooldown(guid, spellId)
	local list = cooldowns[guid] or EMPTY_TABLE
	for i=#list, 1, -1 do
		local data = list[i]
		if data[1] == spellId then
			deleteCooldownSafe(list, i)
		end
	end
end

-- API

function PABR_GetNumCooldowns(guid) -- return numCooldowns
	return #(cooldowns[guid] or EMPTY_TABLE)
end

function PABR_GetCooldownInfo(guid, index) -- return spellId, start, duration, count, maxCount, id
	return unpack((cooldowns[guid] or EMPTY_TABLE)[index])
end

function PABR_WipeCooldowns()
	wipe(cooldowns)
end

-- Handlers

function PABR_CheckCooldownDetection(method, timestamp, sourceGUID, destGUID, spellId)
	sourceGUID = PABR_ResolveGUID(sourceGUID)
	spellId = PABR_MapSpellId(spellId)
	ctx.guid = sourceGUID
	
	local info = PABR_GetSpellInfo(spellId)

	if detectMatch(method, info.detect) then
		local reset = info.reset or EMPTY_TABLE
		local duration = eval(info.cooldown)

		for i=1, #reset do
			deleteCooldown(sourceGUID, reset[i])
		end
		
		if duration > 0 then
			-- Register or update the cooldown!
			local index, data = findCooldown(sourceGUID, spellId)
			if index == 0 then
				insertCooldown(sourceGUID, spellId, timestamp, duration, eval(info.charges))
			else
				updateCooldown(data, timestamp, duration)
			end
		end

		PABR_SetDirty()
		PABR_CheckRecompute()
	end
	
	--print("detect: ", spellId)
end

function PABR_HandleInterrupt(timestamp, sourceGUID, destGUID, spellId, interruptedSpellId)
	-- TODO

	PABR_CheckRecompute()
end

function PABR_HandleDeath(timestamp, guid)
	-- TODO

	PABR_CheckRecompute()
end

function PABR_HandleNetworkCooldown(guid, spellId, remaining, count)
	-- TODO
end

function PABR_CheckCooldownPrune(now)
	if now > NEXT_PRUNE then
		NEXT_PRUNE = now + 1

		for guid, list in pairs(cooldowns) do
			for i=#list, 1, -1 do
				local data = list[i]
				if GetTime() > data[2] + data[3] + HOLD_TIMER then
					deleteCooldownSafe(list, i)
					PABR_SetDirty()
				end
			end

			if #list == 0 then
				PABR_DeleteTable(list)
				cooldowns[guid] = nil
			end
		end
	end
end

function PABR_CheckCooldownCharge(now)
	for id, data in pairs(charging) do
		local overflow = GetTime() - (data[2] + data[3])
		if overflow > 0 and data[4] < data[5] then
			data[4] = data[4] + 1
			if data[4] < data[5] then
				data[2] = now - overflow
			end
			PABR_SetDirty()
		end
	end
end

function PABR_CheckCooldownPeriodic()
	local now = GetTime()
	
	PABR_CheckCooldownPrune(now)
	PABR_CheckCooldownCharge(now)
	
	PABR_CheckRecompute()
end