--[[

Death Knight
250 = Blood
251 = Frost
252 = Unholy

Druid
102 = Balance
103 = Feral
104 = Guardian
105 = Restoration

Hunter
253 = Beast Mastery
254 = Marksmanship
255 = Survival

Mage
62 = Arcane
63 = Fire
64 = Frost

Monk
268 = Brewmaster
270 = Mistweaver
269 = Windwalker

Paladin
65 = Holy
66 = Protection
67 = Retribution

Priest
256 = Discipline
257 = Holy
258 = Shadow

Rogue
259 = Assassination
260 = Combat
261 = Subtlety

Shaman
262 = Elemental
263 = Enhancement
264 = Restoration

Warlock
265 = Affliction
266 = Demonology
267 = Destruction

Warrior
71 = Arms
72 = Fury
73 = Protection

]]

local inspect = { }

local queue = { }

local NEXT_INSPECT = 0

local slots = { "HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot", "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot", "MainHandSlot", "SecondaryHandSlot" }

-- API

function PABR_HasTalent(guid, talentIndex)
	if inspect[guid] then
		return inspect[guid][1][talentIndex] or false
	end
	return false
end

function PABR_HasGlyph(guid, spellId)
	if inspect[guid] then
		return inspect[guid][2][spellId] or false
	end
	return false
end

function PABR_HasItem(guid, itemId)
	if inspect[guid] then
		return inspect[guid][3][itemId] or false
	end
	return false
end

function PABR_GetSpecialization(guid)
	if inspect[guid] then
		return inspect[guid][4]
	end
	return 0
end

function PABR_WipeInspect()
	wipe(inspect)
	wipe(queue)
end

-- Functions

local function checkEnqueue(unit)
	local guid = UnitGUID(unit)
	if guid and not inspect[guid] and not UnitIsUnit(unit, "player") then
		tinsert(queue, guid)
	end
end

local function dequeue(guid)
	for i=#queue, 1, -1 do
		if queue[i] == guid then
			tremove(queue, i)
		end
	end
end

local function beginDataAccess(guid)
	local info = inspect[guid]
	if not info then
		info = { {}, {}, {}, 0 }
		inspect[guid] = info
	else
		wipe(info[1])
		wipe(info[2])
		wipe(info[3])
	end
	return info
end

-- Handlers

function PABR_InspectLocalPlayer()
	PABR_InspectIsReady(UnitGUID("player"))
end

function PABR_InspectIsReady(guid)
	local inspectedUnit = PABR_FindUnit(guid)
	if not inspectedUnit then return; end
	local isInspect = inspectedUnit ~= "player"
	local _, _, classID = UnitClass(inspectedUnit)

	dequeue(guid)

	local info = beginDataAccess(guid)

	for i=1, GetNumTalents() do
		local _, _, _, _, selected = GetTalentInfo(i, isInspect, nil, inspectedUnit, classID)

		if selected then
			info[1][i] = true
		end
	end

	for i=1, GetNumGlyphSockets() do
		local _, _, _, spellId = GetGlyphSocketInfo(i, nil, isInspect, inspectedUnit)

		if spellId then
			info[2][spellId] = true
		end
	end

	for i=1, #slots do
		local itemId = GetInventoryItemID(inspectedUnit, GetInventorySlotInfo(slots[i]))

		if itemId then
			info[3][itemId] = true
		end
	end
	
	if isInspect then
		info[4] = GetInspectSpecialization(inspectedUnit) or 0
	else
		local spec = GetSpecialization()
		if spec then info[4] = GetSpecializationInfo(spec) or 0 end
	end
end

function PABR_ArenaEnemyIsReady(unit)
	assert(unit:sub(1, 5) == "arena")
	
	local id = tonumber(unit:sub(6))
	local guid = UnitGUID(unit)
	if id and guid then
		local info = beginDataAccess(guid)
			
		-- No talent/glyph/item info available obviously
			
		info[4] = GetArenaOpponentSpec(id) or 0
	end
end

function PABR_EnqueueInspect()
	wipe(queue)

	if PABR_Config["EnableInspect"] then
		if IsInRaid() then
			for i=1, GetNumGroupMembers() do
				checkEnqueue("raid"..i)
			end
		else
			for i=1, GetNumGroupMembers() do
				checkEnqueue("party"..i)
			end
		end
	end
end

function PABR_ProcessInspectQueue()
	if GetTime() > NEXT_INSPECT then
		NEXT_INSPECT = GetTime() + 1
		
		PABR_InspectLocalPlayer()
		
		for i=1, #queue do
			local unit = PABR_FindUnit(queue[i])

			if unit and CanInspect(unit) and not (InspectFrame and InspectFrame:IsShown()) then
				NotifyInspect(unit)

				break
			end
		end
	end
end

function PABR_OnNotifyInspect(unit)
	NEXT_INSPECT = GetTime() + 3
end

do
	hooksecurefunc("NotifyInspect", PABR_OnNotifyInspect)
end