local UNITS = { }

do
	tinsert(UNITS, "player")
	tinsert(UNITS, "pet")

	for i=1, 5 do
		tinsert(UNITS, "arena"..i)
		tinsert(UNITS, "party"..i)
	end
	for i=1, 40 do
		tinsert(UNITS, "raid"..i)
	end

	tinsert(UNITS, "playertarget")
	tinsert(UNITS, "pettarget")
	tinsert(UNITS, "target")
	tinsert(UNITS, "targettarget")
	tinsert(UNITS, "focus")
	tinsert(UNITS, "focustarget")
	tinsert(UNITS, "mouseover")
	tinsert(UNITS, "mouseovertarget")

	for i=1, 5 do
		tinsert(UNITS, "arena"..i.."target")
		tinsert(UNITS, "party"..i.."target")
	end
	for i=1, 40 do
		tinsert(UNITS, "raid"..i.."target")
	end
end

local DIRTY = false
local ENABLED = false

local offset = 0

local output = { {}, {} }

local callbacks = { }

local function fireCallback(teamId)
	for i=1, #callbacks do
		if callbacks[i][1] == teamId then
			callbacks[i][2](callbacks[i][3])
		end
	end
end

-- API

function PABR_IsEngineEnabled()
	return ENABLED
end

function PABR_GetTeamNumCooldowns(teamId) -- return numCooldowns
	return #output[teamId]
end

function PABR_GetTeamCooldownInfo(teamId, index) -- return id, spellId, guid, name, cooldownStart, cooldownDuration, cooldownCount, cooldownMaxCount, effectStart, effectDuration
	return unpack(output[teamId][index])
end

function PABR_RegisterUpdateCallback(teamId, func, arg)
	tinsert(callbacks, {teamId, func, arg})
end

function PABR_FindUnit(guid)
	if guid then
		for i=1, #UNITS do
			if UnitGUID(UNITS[i]) == guid then
				return UNITS[i]
			end
		end
	end
	return nil
end

function PABR_GetTime(timestamp)
	return timestamp + offset
end

-- Functions

local available = { }

function PABR_CreateTable()
	local table
	if #available > 0 then
		table = available[#available]
		tremove(available, #available)
	else
		table = { }
	end
	return table
end

function PABR_DeleteTable(table)
	wipe(table)
	tinsert(available, table)
end

function PABR_DeepCopy(t)
	local res = t
	if type(t) == "table" then
		res = {}
		for k, v in pairs(t) do
			if type(v) == "table" then
				v = PABR_DeepCopy(v)
			end
			res[k] = v
		end
	end
	return res
end

-- Handlers

function PABR_OnCombatLogEvent(timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, ...)
	if not PABR_IsEngineEnabled() or not sourceGUID or not destGUID then return; end

	if offset == 0 then
		offset = GetTime() - timestamp
	end

	if strsub(event, 1, 5) == "SPELL" then
		local spellId, spellName, spellSchool = ...

		if event == "SPELL_AURA_APPLIED" then
			PABR_OnEffectGain(timestamp, sourceGUID, destGUID, spellId)
			PABR_CheckCooldownDetection("effectup", timestamp, sourceGUID, destGUID, spellId)

		elseif event == "SPELL_AURA_REMOVED" then
			PABR_OnEffectFade(timestamp, sourceGUID, destGUID, spellId)
			PABR_CheckCooldownDetection("effectdown", timestamp, sourceGUID, destGUID, spellId)

  		elseif event == "SPELL_CAST_SUCCESS" or event == "SPELL_SUMMON" or event == "SPELL_RESURRECT" then
			if event == "SPELL_SUMMON" then
				PABR_NotifySummon(sourceGUID, destGUID)
			end
			PABR_CheckCooldownDetection("cast", timestamp, sourceGUID, destGUID, spellId)

		elseif event == "SPELL_DAMAGE" or event == "SPELL_HEAL" then
			PABR_CheckCooldownDetection("hit", timestamp, sourceGUID, destGUID, spellId)

		elseif event == "SPELL_MISSED" then 
			PABR_CheckCooldownDetection("miss", timestamp, sourceGUID, destGUID, spellId)

		elseif event == "SPELL_INTERRUPT" then
			local extraSpellId, extraSpellName = select(4, ...)
			PABR_HandleInterrupt(timestamp, sourceGUID, destGUID, spellId, extraSpellId)
		end

	elseif event == "UNIT_DIED" or event == "UNIT_DESTROYED" then
		PABR_HandleDeath(timestamp, destGUID)
	end
end

function PABR_ResetEngine()
	PABR_WipeEffects()
	PABR_WipeEnemyTeam()
	PABR_WipeSummon()
	PABR_WipeCooldowns()
	PABR_WipeInspect()
end

function PABR_ToggleEngine(enable)
	ENABLED = enable
	PABR_UpdatePlayerTeam()
end

function PABR_SetDirty()
	DIRTY = true
end

function PABR_CheckRecompute()
	if DIRTY then
		DIRTY = false

		for i=1, #output do
			local out = output[i]

			for k, v in pairs(out) do
				out[k] = nil

				PABR_DeleteTable(v)
			end

			if PABR_IsEngineEnabled() then
				if PABR_Config["EnableTest"] then
					for j=1, 100 do
						local t = PABR_CreateTable()
						t[1] = j
						t[2] = 42292
						t[3] = UnitGUID("player")
						t[4] = UnitName("player")
						t[5] = GetTime()
						t[6] = 60
						t[7] = 0
						t[8] = 0
						if math.fmod(j, 2) == 0 then
							t[9] = 0
							t[10] = 0
						else
							t[9] = GetTime()
							t[10] = 30
						end
						tinsert(out, t)
					end
				else
					for j=1, PABR_GetTeamSize(i) do
						local guid, name = PABR_GetTeamMember(i, j)

						if guid ~= UnitGUID("player") or PABR_Config["ShowLocalPlayer"] then
							for k=1, PABR_GetNumCooldowns(guid) do
								local spellId, cooldownStart, cooldownDuration, count, maxCount, id = PABR_GetCooldownInfo(guid, k)

								if PABR_IsSpellShown(spellId) then
									local effectStart, effectDuration = PABR_GetEffectInfo(guid, spellId)

									local t = PABR_CreateTable()
									t[1] = id
									t[2] = spellId
									t[3] = guid
									t[4] = name
									t[5] = cooldownStart
									t[6] = cooldownDuration
									t[7] = count
									t[8] = maxCount
									t[9] = effectStart
									t[10] = effectDuration
									tinsert(out, t)
								end
							end
						end
					end
				end
			end

			fireCallback(i)
		end
	end
end