local PABR = CreateFrame("Frame", "PABR")
local PABR_VERSION = 1
local PABR_FULL = "|cff40c0ffParty Ability Bars Reloaded|r"
local PABR_SHORT = "|cff40c0ffPABR|r"

-- Events

PABR:SetScript("OnEvent", function(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		PABR_OnCombatLogEvent(...)

	elseif event == "UNIT_AURA" then
		local unit = ...
		PABR_UpdateEffect(unit)

	elseif event == "UNIT_TARGET" then
		local unit = ...
		PABR_CheckEnemyUnit(unit.."target")

	elseif event == "CHAT_MSG_ADDON" then
		PABR_HandleNetwork(...)

	elseif event == "PLAYER_ENTERING_WORLD" and PABR_IsConfigReady() then
		PABR_ResetEngine()
		PABR_CheckEnable()

	elseif event == "VARIABLES_LOADED" then
		local result = PABR_InitConfig()
		if result == "init" then
			print(PABR_SHORT.." - Configuration initialized.")
		elseif result == "update" then
			print(PABR_SHORT.." - Configuration has changed, please check it!")
		elseif result == "ok" then
			print(PABR_SHORT.." - Configuration loaded.")
		end
		PABR_Config_RefreshAll()
		PABR_InitNetwork()
		SLASH_PABR1 = "/pabr"
		SlashCmdList["PABR"] = function(msg)
			InterfaceOptionsFrame_OpenToCategory("PAB Reloaded")
		end
		print(("%s ready (version %d)."):format(PABR_FULL, PABR_VERSION))
		PABR_CheckEnable()

	elseif event == "GROUP_ROSTER_UPDATE" then
		PABR_CheckEnable()

	elseif event == "ARENA_OPPONENT_UPDATE" then
		local unit, subEvent = ...
		if subEvent == "seen" then
			PABR_CheckEnemyUnit(unit)
			PABR_ArenaEnemyIsReady(unit)
		end

	elseif event == "INSPECT_READY" then
		local guid = ...
		PABR_InspectIsReady(guid)
	end
end)

PABR:SetScript("OnUpdate", function(self, elapsed)
	PABR_CheckPets()
	PABR_CheckCooldownPeriodic()
	PABR_ProcessInspectQueue()
end)

do
	PABR:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	PABR:RegisterEvent("UNIT_AURA")
	PABR:RegisterEvent("UNIT_TARGET")
	PABR:RegisterEvent("CHAT_MSG_ADDON")
	PABR:RegisterEvent("PLAYER_ENTERING_WORLD")
	PABR:RegisterEvent("VARIABLES_LOADED")
	PABR:RegisterEvent("GROUP_ROSTER_UPDATE")
	PABR:RegisterEvent("ARENA_OPPONENT_UPDATE")
	PABR:RegisterEvent("INSPECT_READY")
end

function PABR_CheckEnable()
	local enableMask = 0
	local isArena, isRegistered = IsActiveBattlefieldArena()
	local _, instanceType = GetInstanceInfo()

	if PABR_Config["EnableTest"] then
		enableMask = 3
	elseif isArena then
		enableMask = PABR_Config["EnableArena"]
	elseif instanceType == "pvp" then
		enableMask = (isRegistered and PABR_Config["EnableRatedBg"]) or (not isRegistered and PABR_Config["EnableBg"])
	elseif IsInRaid() then
		enableMask = PABR_Config["EnableRaid"]	
	elseif GetNumGroupMembers() > 0 then
		enableMask = PABR_Config["EnableParty"]
	else
		enableMask = PABR_Config["EnableSolo"]
	end

	PABR_ToggleUI(enableMask)
	PABR_ToggleEngine(enableMask > 0)
end