local filters = {
	{"ALL", "All"},
	{"ITEM", "Items/Racials/Tradeskills"},
	--{"RACIAL", "Racials"},
	{"SYMBIOSIS", "Symbiosis"},
	{"PET", "Pets"},	
	{"", "____ CLASSES ____"},
	{"DEATHKNIGHT"},
	{"DRUID"},
	{"HUNTER"},
	{"MAGE"},
	{"MONK"},
	{"PALADIN"},
	{"PRIEST"},
	{"ROGUE"},
	{"SHAMAN"},
	{"WARLOCK"},
	{"WARRIOR"},
	{"", "____ GROUPS ____"},
	{"important", "Important"},
	{"dispel", "Dispel"},
	{"control", "Crowd Control"},
	{"disable", "Disarm/Silence"},
	{"stun", "Stun"},
	{"interrupt", "Interrupt"},
	{"attack", "Attack"},
	{"defense", "Defense"},
	{"utility", "Utility"},
	{"resurrect", "Resurrect"},
}

local orders = { }

do
	for index, info in ipairs(filters) do
		local className = LOCALIZED_CLASS_NAMES_MALE[info[1]]
		if className then
			info[2] = className
		end
		orders[info[1]] = index
	end
end

local function sortScore(s)
	local i = PABR_GetSpellInfo(s)
	return s - orders[i.class] * 1000000 - orders[i.category] * 100000000
end

local function sortFunc(s1, s2)
	return sortScore(s1) > sortScore(s2)
end

local SPELLS_PER_ROW = 3
local ROWS_PER_PAGE = 12
local ACTIVE_FILTER = "ALL"

local function passFilter(button)
	local info = PABR_GetSpellInfo(button:GetID())
	return ACTIVE_FILTER == "ALL" or info.category == ACTIVE_FILTER or tContains(info.tags, ACTIVE_FILTER)
end

-- Functions

function PABR_SpellConfigPanel_OnLoad(self)
	self.name = "Spells"
	self.parent = "PAB Reloaded"
	self.okay = PABR_SpellConfigPanel_Okay
	self.cancel = PABR_SpellConfigPanel_Refresh
	self.default = PABR_SpellConfigPanel_Default
	InterfaceOptions_AddCategory(self)
	
	PABR_SpellConfigPanel_ShowAllButton:SetText("Show all")
	PABR_SpellConfigPanel_HideAllButton:SetText("Hide all")
	
	local list = PABR_GetAllSpells()
	table.sort(list, sortFunc)

	self.spells = { }
	for i=1, #list do
		local id = list[i]
		local button = CreateFrame("CheckButton", self:GetName().."_SpellPanel_Entry"..i, self.spellPanel, "PABR_SpellEntryTemplate")
		local name = GetSpellInfo(id)
		local info = PABR_GetSpellInfo(id)
		local icon = PABR_GetSpellIcon(id, false)
		local coord = CLASS_BUTTONS[info.class]
		if coord then
			button.class:Show()
			button.class:SetTexCoord(unpack(coord))
		else
			button.class:Hide()
		end
		button.icon:SetTexture(icon)
		button.text:SetText(name)
		button:SetID(id)
		button:Hide()

		self.spells[i] = button
	end
	self.filteredSpells = { }
end

function PABR_SpellConfigPanel_Okay(self)
	local buttons = self.spells
	for i=1, #buttons do
		PABR_SetSpellShown(buttons[i]:GetID(), buttons[i]:GetChecked())
	end
end

function PABR_SpellConfigPanel_Default(self)
	PABR_ResetActiveProfile("spells")

	PABR_SpellConfigPanel_Refresh(self)
end

function PABR_SpellConfigPanel_Refresh(self)
	PABR_SpellConfigPanel_Resync(self)
	PABR_SpellConfigPanel_Refilter(self)
end

function PABR_SpellConfigPanel_Resync(self)
	local buttons = PABR_SpellConfigPanel.spells
	for i=1, #buttons do
		local button = buttons[i]
		local enabled = PABR_IsSpellShown(button:GetID())
		button:SetChecked(enabled)
		button:UpdateColor()
	end
end

function PABR_SpellConfigPanel_Refilter(self)
	local buttons = PABR_SpellConfigPanel.spells
	local visible = PABR_SpellConfigPanel.filteredSpells

	-- Recompute filtered spells
	wipe(visible)
	for i=1, #buttons do
		local button = buttons[i]
		if passFilter(button) then
			tinsert(visible, button)
		else
			button:Hide()
		end
	end

	-- Show active filter
	local dropDown = _G[self:GetName().."_FilterDropDown"]
	UIDropDownMenu_SetWidth(dropDown, 164)
	UIDropDownMenu_Initialize(dropDown, PABR_SpellConfigPanel_FilterDropDown_Initialize)
	UIDropDownMenu_SetSelectedValue(dropDown, ACTIVE_FILTER)
	
	-- Update list since the filter changed
	PABR_SpellConfigPanel_SpellPanel_Update()
end

function PABR_SpellConfigPanel_SpellPanel_Update()
	local visible = PABR_SpellConfigPanel.filteredSpells
	local rowOffset = FauxScrollFrame_GetOffset(PABR_SpellConfigPanel_SpellPanel_ScrollFrame)
	local numRows = floor((#visible - 1) / SPELLS_PER_ROW) + 1
	local shown = 0

	for i=1, #visible do
		local button = visible[i]
		if i > rowOffset * SPELLS_PER_ROW and shown < ROWS_PER_PAGE * SPELLS_PER_ROW then
			button:SetPoint("TOPLEFT", PABR_SpellConfigPanel_SpellPanel, "TOPLEFT", 8 + math.fmod(shown, 3) * 184, -8 - floor(shown / 3) * 40)
			button:Show()
			shown = shown + 1
		else
			button:Hide()
		end
	end	

	FauxScrollFrame_Update(PABR_SpellConfigPanel_SpellPanel_ScrollFrame, numRows, ROWS_PER_PAGE, 40)
end

function PABR_SpellConfigPanel_FilterDropDown_Initialize(self)
	local selectedValue = UIDropDownMenu_GetSelectedValue(self)
	local info = UIDropDownMenu_CreateInfo()

	for index, filter in ipairs(filters) do
		info.value = filter[1]
		info.text = filter[2]
		info.disabled = #filter[1] == 0
		info.checked = info.value == selectedValue
		info.func = PABR_SpellConfigPanel_FilterDropDown_OnClick
		UIDropDownMenu_AddButton(info)
	end
end

function PABR_SpellConfigPanel_FilterDropDown_OnClick(self)
	ACTIVE_FILTER = self.value
	PABR_SpellConfigPanel_Refilter(PABR_SpellConfigPanel)
end

function PABR_SpellConfigPanel_SetAll(enabled)
	local visible = PABR_SpellConfigPanel.filteredSpells
	for i=1, #visible do
		local button = visible[i]
		button:SetChecked(enabled)
		button:UpdateColor()
	end
end