local checkButtons = {
	"EnableTest",
	"UILocked",

	"ShowEffect",
	"ShowPortrait",
	"ShowLocalPlayer",
	"FlashTargetCooldowns",
	"EnableInspect",
}

local displayDropDowns = {
	"EnableArena",
	"EnableRatedBg",
	"EnableBg",
	"EnableRaid",
	"EnableParty",
	"EnableSolo",
}

local displays = {
	{0, "Disable user interface"},
	{1, "Show my team cooldowns"},
	{2, "Show enemy team cooldowns"},
	{3, "Show both teams cooldowns"},
}

-- Functions

function PABR_Config_SaveAll()
	PABR_ConfigPanel_Okay(PABR_ConfigPanel)
	PABR_SpellConfigPanel_Okay(PABR_SpellConfigPanel)
end

function PABR_Config_RefreshAll()
	PABR_ConfigPanel_Refresh(PABR_ConfigPanel)
	PABR_SpellConfigPanel_Refresh(PABR_SpellConfigPanel)
	PABR_ProfileConfigPanel_Refresh(PABR_ProfileConfigPanel)
end

function PABR_ConfigPanel_OnLoad(self)
	self.name = "PAB Reloaded"
	self.okay = PABR_ConfigPanel_Okay
	self.cancel = PABR_ConfigPanel_Refresh
	self.default = PABR_ConfigPanel_Default
	InterfaceOptions_AddCategory(self)

	PABR_ConfigPanel_EnableTestCheckButton:SetText("|cff00ff00Test|r user interface")
	PABR_ConfigPanel_UILockedCheckButton:SetText("|cff00ff00Lock|r user interface")

	PABR_ConfigPanel_ShowEffectCheckButton:SetText("Show effect duration before showing cooldown")
	--PABR_ConfigPanel_ShowPortraitCheckButton:SetText("Show character portraits (recommended for big teams)")
	PABR_ConfigPanel_ShowPortraitCheckButton:SetText("Show character portraits |cffffff00[not yet implemented]|r")
	PABR_ConfigPanel_ShowLocalPlayerCheckButton:SetText("Show your own character cooldowns")
	PABR_ConfigPanel_FlashTargetCooldownsCheckButton:SetText("Make target's cooldowns flash")
	PABR_ConfigPanel_EnableInspectCheckButton:SetText("Enable inspections (better simulations for your team)")

	PABR_ConfigPanel_EnableArenaText:SetText("When I am in an arena...")
	PABR_ConfigPanel_EnableRatedBgText:SetText("When I am in a rated battlefield...")
	PABR_ConfigPanel_EnableBgText:SetText("When I am in a normal battlefield...")
	PABR_ConfigPanel_EnableRaidText:SetText("When I am in a raid...")
	PABR_ConfigPanel_EnablePartyText:SetText("When I am in a party...")
	PABR_ConfigPanel_EnableSoloText:SetText("When I am alone...")
end

function PABR_ConfigPanel_Okay(self)
	for index, option in ipairs(checkButtons) do
		PABR_Config[option] = not not _G[self:GetName().."_"..option.."CheckButton"]:GetChecked()
	end

	for index, option in ipairs(displayDropDowns) do
		PABR_Config[option] = UIDropDownMenu_GetSelectedValue(_G[self:GetName().."_"..option.."DropDown"])
	end

	-- Reevaluate activation of the mod
	PABR_CheckEnable()
end

function PABR_ConfigPanel_Default(self)
	PABR_ResetActiveProfile("config")

	-- Reevaluate activation of the mod
	PABR_CheckEnable()
	
	PABR_ConfigPanel_Refresh(self)
end

function PABR_ConfigPanel_Refresh(self)
	for index, option in ipairs(checkButtons) do
		_G[self:GetName().."_"..option.."CheckButton"]:SetChecked(PABR_Config[option])
	end

	for index, option in ipairs(displayDropDowns) do
		local dropDown = _G[self:GetName().."_"..option.."DropDown"]

		UIDropDownMenu_SetWidth(dropDown, 200)
		UIDropDownMenu_Initialize(dropDown, PABR_ConfigPanel_DisplayDropDown_Initialize)
		UIDropDownMenu_SetSelectedValue(dropDown, PABR_Config[option])
	end
end

function PABR_ConfigPanel_DisplayDropDown_Initialize(self)
	local selectedValue = UIDropDownMenu_GetSelectedValue(self)
	local info = UIDropDownMenu_CreateInfo()

	for index, option in ipairs(displays) do
		info.value = option[1]
		info.text = option[2]
		info.checked = info.value == selectedValue
		info.func = PABR_ConfigPanel_DisplayDropDown_OnClick
		info.arg1 = self
		UIDropDownMenu_AddButton(info)
	end
end

function PABR_ConfigPanel_DisplayDropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(self.arg1, self.value)
end