-- ========================================================
--                          Data
-- ========================================================

local EMPTY_TABLE = {}

local db = {
	-- Default entry
	[0] = {
		category = "ITEM",
		class = "ITEM",
		detect = "none", -- Possible values: cast, miss, hit, effectup, effectdown
		cooldown = 0,
		tags = EMPTY_TABLE,
	}
}

local trinketIcons = {
	["Neutral"]  = {"Interface\\Icons\\inv_misc_rune_06", "Interface\\Icons\\inv_misc_rune_06"}, -- Placeholder / should never happen in current game
	["Alliance"] = {"Interface\\Icons\\inv_jewelry_trinketpvp_01", "Interface\\Icons\\inv_jewelry_trinketpvp_02"},
	["Horde"]    = {"Interface\\Icons\\inv_jewelry_trinketpvp_02", "Interface\\Icons\\inv_jewelry_trinketpvp_01"},
}

-- FOR TEST ONLY
--[[
do
	local count = 0
	for i=1, 100000 do
		if count < 40 then
			local name = GetSpellInfo(i)
			if name then
				db[i] = {
					class = "DRUID",
					cooldown = 30,
					important = math.fmod(count, 7) == 0,
				}
				count = count + 1
			end
		else
			break
		end
	end
end
]]

-- ========================================================
--                         Functions
-- ========================================================

function PABR_Insert(category, table)
	for k, v in pairs(table) do
		v.category = category
		v.class = v.class or category
		v.tags = v.tags or EMPTY_TABLE
		db[k] = v
	end
end

function PABR_MapSpellId(spellId)
	local info = db[spellId]
	if info and info.redirect then return info.redirect; end
	return spellId
end

function PABR_GetSpellInfo(spellId)
	return db[spellId] or db[0]
end

function PABR_GetAllSpells()
	local list = { }
	for k, v in pairs(db) do
		if k > 0 and not v.redirect then tinsert(list, k); end
	end
	return list
end

function PABR_GetTaggedSpells(tag)
	local list = { }
	for k, v in pairs(db) do
		if tContains(v.tags, tag) then tinsert(list, k); end
	end
	return list
end

function PABR_GetSpellIcon(spellId, enemy)
	local info = PABR_GetSpellInfo(spellId)
	if info.trinket then
		return trinketIcons[UnitFactionGroup("player")][(enemy and 2) or 1]
	else
		return select(3, GetSpellInfo(spellId))
	end
end